<!-- 
Copyright (C) 2006 Daniel Webb

This file is part of bot-trap.

bot-trap is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

bot-trap is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with bot-trap; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-->

<?php
function is_blacklisted ($ip)
{
  $badbot = 0;
  global $blacklist_file, $htaccess_file;

  /* Create .htaccess file in / if it's not there already */
  if (!is_file($htaccess_file)) 
    { 
      touch($htaccess_file);
      if (!is_file($htaccess_file)) 
        { 
          echo "<p>Can't create $htaccess_file, do you have permissions?</p>";
        }
      $fp = fopen($htaccess_file, 'w');
      fwrite($fp, "ErrorDocument 403 /bot-trap/forbid.php\n");
      fclose($fp);
    }

  /* Create blacklist file if it's not there already */
  if (!is_file($blacklist_file)) 
    { 
      touch($blacklist_file); 
      if (!is_file($blacklist_file)) 
        {
          echo "<p>Can't create $blacklist_file, do you have permissions?</p>";
        }
    }

  /* scan the blacklist file to see if this IP address is there */
  $fp = fopen($blacklist_file, "r") or die ("Error opening file $blacklist_file <br>\n");
  if (flock($fp, LOCK_SH)) 
    {
      while ($line = fgets($fp,255))  
        {
          if (ereg($ip,$line)) { $badbot++; }
        }
      flock($fp, LOCK_UN); 
    }
  else 
    {
      echo "<p>Can't get lock for $blacklist_file</p>";
    }
  fclose($fp);
  return $badbot;
}
?> 

<?php
function add_blacklist ($ip, $req_method, $req_uri, $server_protocol,
    $http_referrer, $http_user_agent)
{
  global $blacklist_file, $htaccess_file;
  global $alert_email_to, $alert_email_from;
  global $mail_on_add;

  /* Check to make sure they aren't blacklisted already */
  /* Note: this function also makes sure $blacklist_file and $htaccess_file */
  /*       exist via is_blacklisted() */
  if (is_blacklisted($ip)) { return; }

  /* append bad bot address data to blacklist log file: */
  $timestamp = time();
  $datum = date("Y-m-d (D) H:i:s",$timestamp);
  if (!is_file("$blacklist_file"))
    {
      echo "<p>Can't open $blacklist_file, do you have permissions to it?</p>";
      return;
    }
  $fp = fopen($blacklist_file,'a+');
  if (flock($fp, LOCK_EX))
    {
      fwrite($fp,"$ip - - [$datum] \"$req_method $req_uri $server_protocol\" $http_referrer $http_user_agent\n");
      flock($fp, LOCK_UN);
    }
  else
    {
      echo "<p>Couldn't get lock for $blacklist_file</p>";
    }
  fclose($fp);
  
  /* append to .htaccess file for automatic denial: */
  if (!is_file("$htaccess_file"))
    {
      echo "<p>Can't open $htaccess_file, do you have permissions to it?</p>";
      return;
    }
  $fp = fopen($htaccess_file,'a+');
  if (flock($fp, LOCK_EX))
    {
      fwrite($fp,"Deny from $ip\n");
      flock($fp, LOCK_UN);
    }
  else
    {
      echo "<p>Couldn't get lock for $htaccess_file</p>";
    }
  fclose($fp);

  if ($mail_on_add === true)
    {
      /* tell the webmaster about this */
      $hostname = gethostbyaddr($ip); 
      $subject = "danielwebb.us alert: bad robot from $ip ($hostname)";
      $msg = "A bad robot hit $req_uri $datum \n";
      $msg .= "address is $ip, hostname is $hostname, agent is $http_user_agent\n";
      mail($alert_email_to, $subject, $msg, "From: $alert_email_from");
    }
}
?> 

<?php
function remove_line_from_file ($file, $regex)
{
  /* Note: this function should be rewritten to change the file in place similar to: */
  /* http://ca3.php.net/manual/en/function.fopen.php#34575 */
  /* It works OK as is though and I don't have time to mess with it. */

  /* Read good lines into $new_fp */
  if (!is_file("$file"))
    {
      echo "<p>Can't open $file, do you have read permissions to it?</p>";
      return;
    }
  $fp = fopen("$file",'r+');   // Apparently a LOCK_EX needs a "r+" file now, "r" fails
  if (!flock($fp, LOCK_EX))
    {
      echo "<p>Couldn't get lock for $file</p>";
      fclose($fp);
      return;
    }
  $new_fp_name = tempnam("","");
  $new_fp = fopen($new_fp_name, "w");
  if (!is_file($new_fp_name))
    {
      echo "<p>File $new_fp_name doesn't exist, can you create this file?</p>";
      return;
    }
  while ($line = fgets($fp,255))  
    {
      if (!ereg($regex,$line)) { fwrite($new_fp, $line); }
    }
  flock($fp, LOCK_UN);
  fclose($fp);
  fclose($new_fp);

  /* Read $new_fp into $fp (moving will change ownership) */
  $fp = fopen("$file",'w');
  if (!flock($fp, LOCK_EX))
    {
      echo "<p>Couldn't get lock for $file</p>";
      fclose($fp);
      return;
    }
  $new_fp = fopen($new_fp_name, "r");
  while ($line = fgets($new_fp,255))  
    {
      fwrite($fp, $line);
    }
  flock($fp, LOCK_UN);
  fclose($fp);
  fclose($new_fp);
  unlink($new_fp_name);
}
?> 

<?php
function remove_blacklist ($ip, $http_user_agent)
{
  global $blacklist_file, $htaccess_file, $mail_on_remove;
  global $alert_email_to, $alert_email_from;

  if (is_blacklisted($ip))
    {
      remove_line_from_file($blacklist_file, $ip);
      remove_line_from_file($htaccess_file, $ip);

      if ($mail_on_remove === true)
        {
          /* tell the webmaster about this */
          $timestamp = time();
	  $hostname = gethostbyaddr($ip); 
          $datum = date("Y-m-d (D) H:i:s",$timestamp);
          $subject = "danielwebb.us alert: $ip ($hostname) removed from ban list";
          $msg .= "address is $ip, hostname is $hostname, agent is $http_user_agent\n";
          mail($alert_email_to, $subject, $msg, "From: $alert_email_from");
        }
    }
}
?> 

